@@ -1,4 +1,3 @@
-"""CherryPy core request/response handling."""
 
 import Cookie
 import os
@@ -7,21 +6,36 @@ import time
 import types
 
 import cherrypy
-from cherrypy import _cpcgifs
+from cherrypy import _cpcgifs, _cpconfig
 from cherrypy._cperror import format_exc, bare_error
 from cherrypy.lib import http
 
 
 class Hook(object):
-    """A callback and its metadata: failsafe, priority, and kwargs.
+    """A callback and its metadata: failsafe, priority, and kwargs."""
     
-    failsafe: If True, the callback is guaranteed to run even if other
-        callbacks from the same call point raise exceptions.
-    priority: Defines the order of execution for a list of Hooks.
-        Defaults to 50. Priority numbers should be limited to the
-        closed interval [0, 100], but values outside this range are
-        acceptable, as are fractional values.
-    """
+    __metaclass__ = cherrypy._AttributeDocstrings
+    
+    callback = None
+    callback__doc = """
+    The bare callable that this Hook object is wrapping, which will
+    be called when the Hook is called."""
+    
+    failsafe = False
+    failsafe__doc = """
+    If True, the callback is guaranteed to run even if other callbacks
+    from the same call point raise exceptions."""
+    
+    priority = 50
+    priority__doc = """
+    Defines the order of execution for a list of Hooks. Priority numbers
+    should be limited to the closed interval [0, 100], but values outside
+    this range are acceptable, as are fractional values."""
+    
+    kwargs = {}
+    kwargs__doc = """
+    A set of keyword arguments that will be passed to the
+    callable on each call."""
     
     def __init__(self, callback, failsafe=None, priority=None, **kwargs):
         self.callback = callback
@@ -93,6 +107,10 @@ class HookMap(dict):
             newmap[k] = v[:]
         return newmap
     copy = __copy__
+    
+    def __repr__(self):
+        cls = self.__class__
+        return "%s.%s(points=%r)" % (cls.__module__, cls.__name__, self.keys())
 
 
 # Config namespace handlers
@@ -129,57 +147,254 @@ hookpoints = ['on_start_resource', 'before_request_body',
 
 
 class Request(object):
-    """An HTTP request."""
+    """An HTTP request.
+    
+    This object represents the metadata of an HTTP request message;
+    that is, it contains attributes which describe the environment
+    in which the request URL, headers, and body were sent (if you
+    want tools to interpret the headers and body, those are elsewhere,
+    mostly in Tools). This 'metadata' consists of socket data,
+    transport characteristics, and the Request-Line. This object
+    also contains data regarding the configuration in effect for
+    the given URL, and the execution plan for generating a response.
+    """
+    
+    __metaclass__ = cherrypy._AttributeDocstrings
     
     prev = None
+    prev__doc = """
+    The previous Request object (if any). This should be None
+    unless we are processing an InternalRedirect."""
     
     # Conversation/connection attributes
     local = http.Host("localhost", 80)
+    local__doc = \
+        "An http.Host(ip, port, hostname) object for the server socket."
+    
     remote = http.Host("localhost", 1111)
+    remote__doc = \
+        "An http.Host(ip, port, hostname) object for the client socket."
+    
     scheme = "http"
+    scheme__doc = """
+    The protocol used between client and server. In most cases,
+    this will be either 'http' or 'https'."""
+    
     server_protocol = "HTTP/1.1"
+    server_protocol__doc = """
+    The HTTP version for which the HTTP server is at least
+    conditionally compliant."""
+    
     base = ""
+    base__doc = """The (scheme://host) portion of the requested URL."""
     
     # Request-Line attributes
     request_line = ""
+    request_line__doc = """
+    The complete Request-Line received from the client. This is a
+    single string consisting of the request method, URI, and protocol
+    version (joined by spaces). Any final CRLF is removed."""
+    
     method = "GET"
+    method__doc = """
+    Indicates the HTTP method to be performed on the resource identified
+    by the Request-URI. Common methods include GET, HEAD, POST, PUT, and
+    DELETE. CherryPy allows any extension method; however, various HTTP
+    servers and gateways may restrict the set of allowable methods.
+    CherryPy applications SHOULD restrict the set (on a per-URI basis)."""
+    
     query_string = ""
+    query_string__doc = """
+    The query component of the Request-URI, a string of information to be
+    interpreted by the resource. The query portion of a URI follows the
+    path component, and is separated by a '?'. For example, the URI
+    'http://www.cherrypy.org/wiki?a=3&b=4' has the query component,
+    'a=3&b=4'."""
+    
     protocol = (1, 1)
+    protocol__doc = """The HTTP protocol version corresponding to the set
+        of features which should be allowed in the response. If BOTH
+        the client's request message AND the server's level of HTTP
+        compliance is HTTP/1.1, this attribute will be the tuple (1, 1).
+        If either is 1.0, this attribute will be the tuple (1, 0).
+        Lower HTTP protocol versions are not explicitly supported."""
+    
     params = {}
+    params__doc = """
+    A dict which combines query string (GET) and request entity (POST)
+    variables. This is populated in two stages: GET params are added
+    before the 'on_start_resource' hook, and POST params are added
+    between the 'before_request_body' and 'before_handler' hooks."""
     
     # Message attributes
     header_list = []
+    header_list__doc = """
+    A list of the HTTP request headers as (name, value) tuples.
+    In general, you should use request.headers (a dict) instead."""
+    
     headers = http.HeaderMap()
+    headers__doc = """
+    A dict-like object containing the request headers. Keys are header
+    names (in Title-Case format); however, you may get and set them in
+    a case-insensitive manner. That is, headers['Content-Type'] and
+    headers['content-type'] refer to the same value. Values are header
+    values (decoded according to RFC 2047 if necessary). See also:
+    http.HeaderMap, http.HeaderElement."""
+    
     cookie = Cookie.SimpleCookie()
+    cookie__doc = """See help(Cookie)."""
+    
     rfile = None
+    rfile__doc = """
+    If the request included an entity (body), it will be available
+    as a stream in this attribute. However, the rfile will normally
+    be read for you between the 'before_request_body' hook and the
+    'before_handler' hook, and the resulting string is placed into
+    either request.params or the request.body attribute.
+    
+    You may disable the automatic consumption of the rfile by setting
+    request.process_request_body to False, either in config for the desired
+    path, or in an 'on_start_resource' or 'before_request_body' hook.
+    
+    WARNING: In almost every case, you should not attempt to read from the
+    rfile stream after CherryPy's automatic mechanism has read it. If you
+    turn off the automatic parsing of rfile, you should read exactly the
+    number of bytes specified in request.headers['Content-Length'].
+    Ignoring either of these warnings may result in a hung request thread
+    or in corruption of the next (pipelined) request.
+    """
+    
     process_request_body = True
+    process_request_body__doc = """
+    If True, the rfile (if any) is automatically read and parsed,
+    and the result placed into request.params or request.body."""
+    
     methods_with_bodies = ("POST", "PUT")
+    methods_with_bodies__doc = """
+    A sequence of HTTP methods for which CherryPy will automatically
+    attempt to read a body from the rfile."""
+    
     body = None
+    body__doc = """
+    If the request Content-Type is 'application/x-www-form-urlencoded'
+    or multipart, this will be None. Otherwise, this will contain the
+    request entity body as a string; this value is set between the
+    'before_request_body' and 'before_handler' hooks (assuming that
+    process_request_body is True)."""
     
     # Dispatch attributes
     dispatch = cherrypy.dispatch.Dispatcher()
+    dispatch__doc = """
+    The object which looks up the 'page handler' callable and collects
+    config for the current request based on the path_info, other
+    request attributes, and the application architecture. The core
+    calls the dispatcher as early as possible, passing it a 'path_info'
+    argument.
+    
+    The default dispatcher discovers the page handler by matching path_info
+    to a hierarchical arrangement of objects, starting at request.app.root.
+    See help(cherrypy.dispatch) for more information."""
+    
     script_name = ""
+    script_name__doc = """
+    The 'mount point' of the application which is handling this request."""
+    
     path_info = "/"
+    path_info__doc = """
+    The 'relative path' portion of the Request-URI. This is relative
+    to the script_name ('mount point') of the application which is
+    handling this request."""
+
+    login = None
+    login__doc = """
+    When authentication is used during the request processing this is
+    set to 'False' if it failed and to the 'username' value if it succeeded.
+    The default 'None' implies that no authentication happened."""
+    
     app = None
+    app__doc = \
+        """The cherrypy.Application object which is handling this request."""
+    
     handler = None
+    handler__doc = """
+    The function, method, or other callable which CherryPy will call to
+    produce the response. The discovery of the handler and the arguments
+    it will receive are determined by the request.dispatch object.
+    By default, the handler is discovered by walking a tree of objects
+    starting at request.app.root, and is then passed all HTTP params
+    (from the query string and POST body) as keyword arguments."""
+    
     toolmaps = {}
+    toolmaps__doc = """
+    A nested dict of all Toolboxes and Tools in effect for this request,
+    of the form: {Toolbox.namespace: {Tool.name: config dict}}."""
+    
     config = None
+    config__doc = """
+    A flat dict of all configuration entries which apply to the
+    current request. These entries are collected from global config,
+    application config (based on request.path_info), and from handler
+    config (exactly how is governed by the request.dispatch object in
+    effect for this request; by default, handler config can be attached
+    anywhere in the tree between request.app.root and the final handler,
+    and inherits downward)."""
+    
     is_index = None
+    is_index__doc = """
+    This will be True if the current request is mapped to an 'index'
+    resource handler (also, a 'default' handler if path_info ends with
+    a slash). The value may be used to automatically redirect the
+    user-agent to a 'more canonical' URL which either adds or removes
+    the trailing slash. See cherrypy.tools.trailing_slash."""
     
     hooks = HookMap(hookpoints)
+    hooks__doc = """
+    A HookMap (dict-like object) of the form: {hookpoint: [hook, ...]}.
+    Each key is a str naming the hook point, and each value is a list
+    of hooks which will be called at that hook point during this request.
+    The list of hooks is generally populated as early as possible (mostly
+    from Tools specified in config), but may be extended at any time.
+    See also: _cprequest.Hook, _cprequest.HookMap, and cherrypy.tools."""
     
     error_response = cherrypy.HTTPError(500).set_response
-    error_page = {}
-    show_tracebacks = True
-    throws = (KeyboardInterrupt, SystemExit, cherrypy.InternalRedirect)
-    throw_errors = False
+    error_response__doc = """
+    The no-arg callable which will handle unexpected, untrapped errors
+    during request processing. This is not used for expected exceptions
+    (like NotFound, HTTPError, or HTTPRedirect) which are raised in
+    response to expected conditions (those should be customized either
+    via request.error_page or by overriding HTTPError.set_response).
+    By default, error_response uses HTTPError(500) to return a generic
+    error response to the user-agent."""
     
-    namespaces = {"hooks": hooks_namespace,
-                  "request": request_namespace,
-                  "response": response_namespace,
-                  "error_page": error_page_namespace,
-                  # "tools": See _cptools.Toolbox
-                  }
+    error_page = {}
+    error_page__doc = """
+    A dict of {error code: response filename} pairs. The named response
+    files should be Python string-formatting templates, and can expect by
+    default to receive the format values with the mapping keys 'status',
+    'message', 'traceback', and 'version'. The set of format mappings
+    can be extended by overriding HTTPError.set_response."""
+    
+    show_tracebacks = True
+    show_tracebacks__doc = """
+    If True, unexpected errors encountered during request processing will
+    include a traceback in the response body."""
+    
+    throws = (KeyboardInterrupt, SystemExit, cherrypy.InternalRedirect)
+    throws__doc = \
+        """The sequence of exceptions which Request.run does not trap."""
+    
+    throw_errors = False
+    throw_errors__doc = """
+    If True, Request.run will not trap any errors (except HTTPRedirect and
+    HTTPError, which are more properly called 'exceptions', not errors)."""
+    
+    namespaces = _cpconfig.NamespaceSet(
+        **{"hooks": hooks_namespace,
+           "request": request_namespace,
+           "response": response_namespace,
+           "error_page": error_page_namespace,
+           # "tools": See _cptools.Toolbox
+           })
     
     def __init__(self, local_host, remote_host, scheme="http",
                  server_protocol="HTTP/1.1"):
@@ -203,20 +418,13 @@ class Request(object):
         self.namespaces = self.namespaces.copy()
     
     def close(self):
+        """Run cleanup code and remove self from globals. (Core)"""
         if not self.closed:
             self.closed = True
             self.hooks.run('on_end_request')
-            
-            s = (self, cherrypy._serving.response)
-            try:
-                cherrypy.engine.servings.remove(s)
-            except ValueError:
-                pass
-            
-            cherrypy._serving.__dict__.clear()
     
     def run(self, method, path, query_string, req_protocol, headers, rfile):
-        """Process the Request.
+        """Process the Request. (Core)
         
         method, path, query_string, and req_protocol should be pulled directly
             from the Request-Line (e.g. "GET /path?key=val HTTP/1.0").
@@ -305,21 +513,21 @@ class Request(object):
         return cherrypy.response
     
     def respond(self, path_info):
-        """Generate a response for the resource at self.path_info."""
+        """Generate a response for the resource at self.path_info. (Core)"""
         try:
             try:
                 try:
                     if self.app is None:
                         raise cherrypy.NotFound()
                     
-                    # Get the 'Host' header, so we can do HTTPRedirects properly.
+                    # Get the 'Host' header, so we can HTTPRedirect properly.
                     self.process_headers()
                     
                     # Make a copy of the class hooks
                     self.hooks = self.__class__.hooks.copy()
                     self.toolmaps = {}
                     self.get_resource(path_info)
-                    cherrypy._cpconfig._call_namespaces(self.config, self.namespaces)
+                    self.namespaces(self.config)
                     
                     self.hooks.run('on_start_resource')
                     
@@ -328,8 +536,9 @@ class Request(object):
                             self.process_request_body = False
                         
                         if self.process_request_body:
-                            # Prepare the SizeCheckWrapper for the request body
-                            mbs = cherrypy.server.max_request_body_size
+                            # Prepare the SizeCheckWrapper for the req body
+                            mbs = getattr(cherrypy.server,
+                                          "max_request_body_size", 0)
                             if mbs > 0:
                                 self.rfile = http.SizeCheckWrapper(self.rfile, mbs)
                     
@@ -356,6 +565,7 @@ class Request(object):
             self.handle_error(sys.exc_info())
     
     def process_headers(self):
+        """Parse HTTP header data into Python structures. (Core)"""
         self.params = http.parse_query_string(self.query_string)
         
         # Process the headers into self.headers
@@ -392,7 +602,7 @@ class Request(object):
         self.base = "%s://%s" % (self.scheme, host)
     
     def get_resource(self, path):
-        """Find and call a dispatcher (which sets self.handler and .config)."""
+        """Call a dispatcher (which sets self.handler and .config). (Core)"""
         dispatch = self.dispatch
         # First, see if there is a custom dispatch at this URI. Custom
         # dispatchers can only be specified in app.config, not in _cp_config
@@ -418,7 +628,19 @@ class Request(object):
         dispatch(path)
     
     def process_body(self):
-        """Convert request.rfile into request.params (or request.body)."""
+        """Convert request.rfile into request.params (or request.body). (Core)"""
+        if not self.headers.get("Content-Length", ""):
+            # No Content-Length header supplied (or it's 0).
+            # If we went ahead and called cgi.FieldStorage, it would hang,
+            # since it cannot determine when to stop reading from the socket.
+            # See http://www.cherrypy.org/ticket/493.
+            # See also http://www.cherrypy.org/ticket/650.
+            # Note also that we expect any HTTP server to have decoded
+            # any message-body that had a transfer-coding, and we expect
+            # the HTTP server to have supplied a Content-Length header
+            # which is valid for the decoded entity-body.
+            return
+        
         # FieldStorage only recognizes POST, so fake it.
         methenv = {'REQUEST_METHOD': "POST"}
         try:
@@ -441,6 +663,7 @@ class Request(object):
             self.params.update(http.params_from_CGI_form(forms))
     
     def handle_error(self, exc):
+        """Handle the last exception. (Core)"""
         try:
             self.hooks.run("before_error_response")
             if self.error_response:
@@ -453,7 +676,7 @@ class Request(object):
 
 
 def file_generator(input, chunkSize=65536):
-    """Yield the given input (a file object) in chunks (default 64k)."""
+    """Yield the given input (a file object) in chunks (default 64k). (Core)"""
     chunk = input.read(chunkSize)
     while chunk:
         yield chunk
@@ -490,18 +713,53 @@ class Body(object):
 
 
 class Response(object):
-    """An HTTP Response."""
+    """An HTTP Response, including status, headers, and body.
+    
+    Application developers should use Response.headers (a dict) to
+    set or modify HTTP response headers. When the response is finalized,
+    Response.headers is transformed into Response.header_list as
+    (key, value) tuples.
+    """
+    
+    __metaclass__ = cherrypy._AttributeDocstrings
     
     # Class attributes for dev-time introspection.
     status = ""
+    status__doc = """The HTTP Status-Code and Reason-Phrase."""
+    
     header_list = []
+    header_list__doc = """
+    A list of the HTTP response headers as (name, value) tuples.
+    In general, you should use response.headers (a dict) instead."""
+    
     headers = http.HeaderMap()
+    headers__doc = """
+    A dict-like object containing the response headers. Keys are header
+    names (in Title-Case format); however, you may get and set them in
+    a case-insensitive manner. That is, headers['Content-Type'] and
+    headers['content-type'] refer to the same value. Values are header
+    values (decoded according to RFC 2047 if necessary). See also:
+    http.HeaderMap, http.HeaderElement."""
+    
     cookie = Cookie.SimpleCookie()
+    cookie__doc = """See help(Cookie)."""
+    
     body = Body()
+    body__doc = """The body (entity) of the HTTP response."""
+    
     time = None
+    time__doc = """The value of time.time() when created. Use in HTTP dates."""
+    
     timeout = 300
+    timeout__doc = """Seconds after which the response will be aborted."""
+    
     timed_out = False
+    timed_out__doc = """
+    Flag to indicate the response should be aborted, because it has
+    exceeded its timeout."""
+    
     stream = False
+    stream__doc = """If False, buffer the response body."""
     
     def __init__(self):
         self.status = None
@@ -520,12 +778,13 @@ class Response(object):
         self.cookie = Cookie.SimpleCookie()
     
     def collapse_body(self):
+        """Iterate over self.body, replacing it with and returning the result."""
         newbody = ''.join([chunk for chunk in self.body])
         self.body = newbody
         return newbody
     
     def finalize(self):
-        """Transform headers (and cookies) into cherrypy.response.header_list."""
+        """Transform headers (and cookies) into self.header_list. (Core)"""
         try:
             code, reason, _ = http.valid_status(self.status)
         except ValueError, x:
@@ -537,7 +796,7 @@ class Response(object):
         if self.stream:
             if dict.get(headers, 'Content-Length') is None:
                 dict.pop(headers, 'Content-Length', None)
-        elif code < 200 or code in (204, 304):
+        elif code < 200 or code in (204, 205, 304):
             # "All 1xx (informational), 204 (no content),
             # and 304 (not modified) responses MUST NOT
             # include a message-body."
@@ -568,3 +827,4 @@ class Response(object):
         if time.time() > self.time + self.timeout:
             self.timed_out = True
 
+
